#include <owl\applicat.h>
#include <owl\framewin.h>
#include <owl\dc.h>
#include "pr51_1.rc"

//   .
enum TShapeMode {mLine, mPie, mEllipse, mRndRect,
mIcon, mBitmap, mFill, mPixel};

//   .
enum TOptionMode {manual, automatic};

// .
class TApp: public TApplication
{
  public:
	 TApp():TApplication() {}
	 void InitMainWindow();
};

//   .
class TWndw : public  TFrameWindow
{
  public:
	 TWndw (TWindow *parent, const char far *title);
  protected:
			TShapeMode shape;
			TOptionMode operation;
			TClientDC *clientDC;
			TBrush *brush;
			TBitmap *bitmapl;
			TIcon *GDIIcon;
			int timer;

			void SetupWindow();
			void CleanUpWindow();
			void Paint(TDC&, BOOL, TRect&);
			void CmChangeShape(WPARAM Id);
			void CmManual() ;
			void CmAuto();
			void EvLButtonDown (UINT, TPoint &point);
			void EvRButtonDown (UINT, TPoint&);
			void EvTimer(UINT);
			void EvChar(UINT, UINT, UINT);

			//   .
			void CmEnableShapes (TCommandEnabler &commandEnabler);
			void CmEnableManual (TCommandEnabler &commandEnabler);
			void CmEnableAuto (TCommandEnabler &commandEnabler);
  private:
			void DrawObject (TPoint &point);

			DECLARE_RESPONSE_TABLE(TWndw);
};

DEFINE_RESPONSE_TABLE1 (TWndw, TFrameWindow)
	  EV_WM_LBUTTONDOWN,
	  EV_WM_RBUTTONDOWN,
	  EV_WM_TIMER,
	  EV_WM_CHAR,

	  EV_COMMAND_AND_ID (CM_PIXEL, CmChangeShape),
	  EV_COMMAND_AND_ID (CM_LINES, CmChangeShape),
	  EV_COMMAND_AND_ID (CM_PIE, CmChangeShape),
	  EV_COMMAND_AND_ID (CM_ELLIPSE, CmChangeShape),
	  EV_COMMAND_AND_ID (CM_RNDRECT, CmChangeShape),
	  EV_COMMAND_AND_ID (CM_ICON, CmChangeShape),
	  EV_COMMAND_AND_ID (CM_BITMAP, CmChangeShape),
	  EV_COMMAND_AND_ID (CM_FILL, CmChangeShape),

	  EV_COMMAND (CM_MANUAL, CmManual),
	  EV_COMMAND (CM_AUTO, CmAuto),

	  EV_COMMAND_ENABLE (CM_PIXEL, CmEnableShapes),
	  EV_COMMAND_ENABLE (CM_LINES, CmEnableShapes),
	  EV_COMMAND_ENABLE (CM_PIE, CmEnableShapes),
	  EV_COMMAND_ENABLE (CM_ELLIPSE, CmEnableShapes),
	  EV_COMMAND_ENABLE (CM_RNDRECT, CmEnableShapes),
	  EV_COMMAND_ENABLE (CM_ICON, CmEnableShapes),
	  EV_COMMAND_ENABLE (CM_BITMAP, CmEnableShapes),
	  EV_COMMAND_ENABLE (CM_FILL, CmEnableShapes),
	  EV_COMMAND_ENABLE (CM_MANUAL, CmEnableManual),
	  EV_COMMAND_ENABLE (CM_AUTO, CmEnableAuto),
END_RESPONSE_TABLE;

// TWndw::TWndw()
//    .
TWndw::TWndw(TWindow *parent, const char far *title):
			 TFrameWindow (parent, title)
{
  //     .
  Attr.X = 20;
  Attr.Y = 20;
  Attr.W = 500;
  Attr.H = 400;
  //    .
  AssignMenu(MENU_1);
  //   .
  SetIcon (GetApplication(), GDIICON);
  //   
  //    .
  shape = mLine;
  operation = manual;
  //    .
  randomize();
}

// TWndw::SetupWindow()
//  ,   ,
//    TFrameWindow,
//   .
void TWndw::SetupWindow()
{
  //    SetupWindow()  . 
  TFrameWindow::SetupWindow();
  //  DC ( )  -
  // .  ,   
  //  DC      
  //        .
  // ,      
  //   .
  clientDC = new TClientDC(HWindow);
  //      
  // - .
  brush = new TBrush(TColor::LtGreen);
  clientDC->SelectObject (*brush);
  //    .
  HINSTANCE instance = *GetApplication();
  //      .
  bitmapl = new TBitmap (instance, BITMAP_1);
  GDIIcon = new TIcon (instance, (TResId)GDIICON);
  //  Windows-.
  timer = SetTimer (1, 10, 0);
  if (timer == 0)
		MessageBox (" !", "", MB_OK);
}

// TWndw::CleanUpWindow()
//    , 
//  TFrameWindow.   
//   .
void TWndw::CleanUpWindow()
{
  //  GDI-  DC.
  delete brush;
  delete bitmapl;
  delete GDIIcon;
  delete clientDC;
  //    . 
  KillTimer(timer);
}

// TWndw::Paint()
//  ,    
// WM_PAINT,   , 
//    .
void TWndw::Paint(TDC&, BOOL, TRect&)
{
  //    DC.
  clientDC->SaveDC();
  //   .
  clientDC->SelectStockObject(GRAY_BRUSH);
  //    . 
  TRect rect = GetClientRect();
  //     . 
  clientDC->Rectangle(rect);
  //  DC   .
  clientDC->RestoreDC();
}

// TWndw::CmChangeShape()
//         Shape,
//      , 
//  .
void TWndw::CmChangeShape(WPARAM Id)
{
  //      
  //   .
  switch (Id)
  {
    case CM_PIXEL  : shape = mPixel; break;
    case CM_LINES  : shape = mLine;  break;
    case CM_PIE    : shape = mPie;   break;
    case CM_ELLIPSE: shape = mEllipse; break;
    case CM_RNDRECT: shape = mRndRect; break;
    case CM_ICON   : shape = mIcon;  break;
    case CM_BITMAP : shape = mBitmap;break;
    case CM_FILL   : shape = mFill;  break;
  }
}

// TWndw::CmManual()
//   ,    
//  Manual   Option,  
//    .
void TWndw::CmManual()
{
  operation = manual;
}

// TWndw::CmAuto()
//   ,   
//   Automatic   Option, 
//     .
void TWndw::CmAuto()
{
  operation = automatic;
}

// TWndw::EvLButtonDown()
//        
//       
//     .
void TWndw::EvLButtonDown(UINT, TPoint &point)
{
  if (operation == manual) DrawObject (point);
}

// TWndw::EvRButtonDown()
//        
// ,  Invalidate    
//  .
void TWndw::EvRButtonDown(UINT, TPoint&)
{
  Invalidate();
}

// TWndw::EvChar()
//      WM_CHAR,
//  Windows     
//   .
void TWndw::EvChar(UINT, UINT, UINT)
{
  Invalidate ();
}

// TWndw::EvTimer()
//      WM_TIMER.
void TWndw::EvTimer(UINT)
{
  //      
  //   ,     .
  if ((operation == automatic) && (!IsIconic()))
  {
    //     .
	 TRect rect = GetClientRect();
    //     . 
    int xpos = random(rect.right); 
    int ypos = random(rect.bottom);
    //  .
    DrawObject (TPoint(xpos, ypos));
  }
}

// TWndw::DrawObject()
//       
//   ,    TPoint.
void TWndw::DrawObject(TPoint  &point) 
{
  BITMAP  bm;
  TColor color;
  switch (shape) 
  {
    case mPixel   : //     . 
      clientDC->SetPixel (point, TColor::LtRed); break;
	 case mLine    : //    .
      clientDC->LineTo (point); break;
    case mPie     : //      . 
      clientDC->Pie (point.x, point.y, point.x+40, point.y+40, 
              point.x+40, point.y+20, point.x+20, point.y+40); break;
    case mEllipse : //     . 
      clientDC->Ellipse (point.x, point.y, point.x+40, point.y+30); break;
    case mRndRect : //    
                    //    . 
		clientDC->RoundRect (point.x, point.y, point.x+40, point.y+30, 20, 20); break;
    case mFill    :   //      .
      color  = clientDC->GetPixel (point); 
                      //    . 
		clientDC->ExtFloodFill (point, color, FLOODFILLSURFACE); break;
    case mIcon    :   //       .
      clientDC->DrawIcon(point, *GDIIcon); break;
    case mBitmap  :  //    
                     //  ,  
                     //   . 
      TMemoryDC memDC (*clientDC); 
                     //    
                     //    . 
      memDC.SelectObject (*bitmapl); 
                     //    . 
      bitmapl->GetObject (bm); 
                     //   
                     //    . 
      clientDC->BitBlt(point.x, point.y, bm.bmWidth, 
                bm.bmHeight, memDC, 0,0, SRCCOPY);
  }
}

// TWwndw::CmEnableShapes()
//       
//  Shapes.     
//   .
void TWndw::CmEnableShapes (TCommandEnabler &commandEnabler)
{
  TShapeMode shapeMode;
  switch (commandEnabler.Id)
  {
    case CM_PIXEL  : shapeMode = mPixel; break;
    case CM_LINES  : shapeMode = mLine;  break;
    case CM_PIE    : shapeMode = mPie;   break;
    case CM_ELLIPSE: shapeMode = mEllipse; break;
    case CM_RNDRECT: shapeMode = mRndRect; break;
    case CM_ICON   : shapeMode = mIcon;  break;
    case CM_BITMAP : shapeMode = mBitmap;break;
    case CM_FILL   : shapeMode = mFill;  break;
  }
  if (shape == shapeMode)
      commandEnabler.SetCheck(TCommandEnabler::Checked); 
  else 
      commandEnabler.SetCheck(TCommandEnabler::Unchecked);
}

// TWwndw::CmEnableManual()
//      Manual  
//  Option.      
// Manual,      . 
void TWndw::CmEnableManual (TCommandEnabler &commandEnabler)
{
  if (operation == manual)
        commandEnabler.SetCheck(TCommandEnabler::Checked); 
  else
        commandEnabler.SetCheck(TCommandEnabler::Unchecked);
} 

// TWwndw::CmEnableAuto()
//      Automatic
//   Option.    
//  Automatic,   
//   .
void TWndw::CmEnableAuto (TCommandEnabler &commandEnabler)
{
  if (operation == automatic)
       commandEnabler.SetCheck(TCommandEnabler::Checked); 
  else commandEnabler.SetCheck(TCommandEnabler::Unchecked);
}

void TApp::InitMainWindow()
{
  TFrameWindow *wndw = new TWndw (0," -GDI");
  SetMainWindow(wndw);
}

int OwlMain(int,char *[])
{
  return TApp().Run();
}
